@@ -34,6 +34,12 @@ enum auth_request_secured {
 	AUTH_REQUEST_SECURED_TLS,
 };
 
+enum auth_request_cache_result {
+	AUTH_REQUEST_CACHE_NONE,
+	AUTH_REQUEST_CACHE_MISS,
+	AUTH_REQUEST_CACHE_HIT,
+};
+
 /* All auth request fields are exported to auth-worker process. */
 struct auth_request_fields {
         /* user contains the user who is being authenticated.
@@ -118,6 +124,9 @@ struct auth_request {
 	struct auth_request_handler *handler;
         struct auth_master_connection *master;
 
+	/* FIXME: Remove this once mech-oauth2 correctly does the processing */
+	const char *openid_config_url;
+
 	unsigned int connect_uid;
 	unsigned int client_pid;
 	unsigned int id;
@@ -157,6 +166,9 @@ struct auth_request {
 
 	void *context;
 
+	enum auth_request_cache_result passdb_cache_result;
+	enum auth_request_cache_result userdb_cache_result;
+
 	/* this is a lookup on auth socket (not login socket).
 	   skip any proxying stuff if enabled. */
 	bool auth_only:1;
@@ -199,8 +211,6 @@ struct auth_request {
 	/* userdb_* fields have been set by the passdb lookup, userdb prefetch
 	   will work. */
 	bool userdb_prefetch_set:1;
-	/* userdb lookup's results are from cache */
-	bool userdb_result_from_cache:1;
 	bool stats_sent:1;
 	bool policy_refusal:1;
 	bool policy_processed:1;
@@ -232,6 +242,8 @@ void auth_request_unref(struct auth_request **request);
 
 void auth_request_success(struct auth_request *request,
 			  const void *data, size_t data_size);
+void auth_request_fail_with_reply(struct auth_request *request,
+				  const void *final_data, size_t final_data_size);
 void auth_request_fail(struct auth_request *request);
 void auth_request_internal_failure(struct auth_request *request);
 
@@ -304,15 +316,19 @@ void auth_request_proxy_finish_failure(struct auth_request *request);
 
 void auth_request_log_password_mismatch(struct auth_request *request,
 					const char *subsystem);
-int auth_request_password_verify(struct auth_request *request,
-				 const char *plain_password,
-				 const char *crypted_password,
-				 const char *scheme, const char *subsystem);
-int auth_request_password_verify_log(struct auth_request *request,
+enum passdb_result
+auth_request_password_verify(struct auth_request *request,
+			     const char *plain_password,
+			     const char *crypted_password,
+			     const char *scheme, const char *subsystem)
+			     ATTR_WARN_UNUSED_RESULT;
+enum passdb_result
+auth_request_password_verify_log(struct auth_request *request,
 				 const char *plain_password,
 				 const char *crypted_password,
 				 const char *scheme, const char *subsystem,
-				 bool log_password_mismatch);
+				 bool log_password_mismatch)
+				 ATTR_WARN_UNUSED_RESULT;
 enum passdb_result auth_request_password_missing(struct auth_request *request);
 
 void auth_request_get_log_prefix(string_t *str, struct auth_request *auth_request,
