@@ -47,7 +47,11 @@ enum mail_search_arg_type {
 	SEARCH_MAILBOX_GUID,
 	SEARCH_MAILBOX_GLOB,
 	SEARCH_REAL_UID,
-	SEARCH_MIMEPART
+	SEARCH_MIMEPART,
+
+	/* This term is allowed only in SEARCH_OR and SEARCH_SUB sublists.
+	   When it is encountered during the simplification, it must be removed */
+	SEARCH_NIL,
 };
 
 enum mail_search_date_type {
@@ -207,6 +211,8 @@ bool mail_search_args_equal(const struct mail_search_args *args1,
    structs. All the siblings of arg1 and arg2 are also compared. */
 bool mail_search_arg_equals(const struct mail_search_arg *arg1,
 			    const struct mail_search_arg *arg2);
+int mail_search_arg_equals_p(const struct mail_search_arg *const *arg1,
+			     const struct mail_search_arg *const *arg2);
 /* Same as mail_search_arg_equals(), but don't compare siblings. */
 bool mail_search_arg_one_equals(const struct mail_search_arg *arg1,
 				const struct mail_search_arg *arg2);
